#include <Arduino.h>
#include <Wire.h>         
#include "RTClib.h"

#if defined(ARDUINO_ARCH_SAMD)
#define Serial SerialUSB
#endif



#include <SPI.h>
#include <Ethernet.h>
#include <SD.h>
// size of buffer used to capture HTTP requests
#define REQ_BUF_SZ   60


RTC_Millis rtc;
DateTime now;
DateTime vrijemeDetekcije;

int vrijemeAkcije;
String motionDetectedString;


byte mac[] = { 0x90, 0xA2, 0xDA, 0x0D, 0xA5, 0xE9 };
IPAddress ip(161, 53, 64, 150 ); 	// IP address
EthernetServer server(80); 			// create a server at port 80
File webFile;              			// the web page file on the SD card
char HTTP_req[REQ_BUF_SZ] = {0}; 	// buffered HTTP request stored as null terminated string
char req_index = 0;              	// index into HTTP_req buffer
boolean LED_state[4] = {0}; 

const int ledPin = 8; 		//PWM pin for LED fading
const int pirPin = 2; 	    //digital input
const unsigned long motionDelay = 1; //the number of minutes the LEDs stay on if motion is detected

bool ledState = false;
const int onHourAM = 7, onMinuteAM = 0, offHourAM = 7, offMinuteAM =30 ;   //morning time
const int onHourPM = 0, onMinutePM = 0, offHourPM = 0, offMinutePM = 30;  //evening time
bool motionDetected = false;
bool relax=false;
/*used to make the code work for both active high and active low motion sensors*/
//#define pirActiveLow //uncomment if your PIR motion sensors is active high, leave this line if your sensor is active low
#ifdef pirActiveLow
  bool pirSensSig = LOW;
#else
  bool pirSensSig = HIGH;
#endif

void setup()
{
    // disable Ethernet chip
    pinMode(10, OUTPUT);
    digitalWrite(10, HIGH);
    
    Serial.begin(9600);       // for debugging
    rtc.begin(DateTime(F(__DATE__), F(__TIME__)));

    pinMode(ledPin, OUTPUT);
    pinMode(pirPin, INPUT);
    
    // initialize SD card
    Serial.println("Initializing SD card...");
    if (!SD.begin(4)) {
        Serial.println("ERROR - SD card initialization failed!");
        return;    // init failed
    }
    Serial.println("SUCCESS - SD card initialized.");
    // check for index.htm file
    if (!SD.exists("index.htm")) {
        Serial.println("ERROR - Can't find index.htm file!");
        return;  // can't find index file
    }
    Serial.println("SUCCESS - Found index.htm file.");
   
    Ethernet.begin(mac, ip);  // initialize Ethernet device
    server.begin();           // start to listen for clients

    initOn();
    now = rtc.now(); //reads time when motionRead function has ended
}

void loop()
{
  printTime();
  if(now.minute() >= vrijemeAkcije && vrijemeAkcije != -1) {
    Serial.println("Isteklo vrijeme. Gasim... ");
    vrijemeAkcije = -1;
    fadeLEDoff();
    motionDetected = false;
  }
  
  
  if((digitalRead(pirPin)) && (now.hour()>onHourPM) && (now.hour()<onHourAM)) // action time
  {
    Serial.println("PIR motion detect");
    vrijemeDetekcije = rtc.now();
    if(motionDetected == false) {
      fadeLEDon();
      if(motionDetectedString.equals("Čekam...")) motionDetectedString = "";
      motionDetectedString.concat( String(String(vrijemeDetekcije.hour()) + ":" + String(vrijemeDetekcije.minute()) + ":" + String(vrijemeDetekcije.second()) + "\n") );
      Serial.print("String: ");
      Serial.println(motionDetectedString);
    }
    
    motionDetected = true;
    vrijemeAkcije = (now.minute() + motionDelay)%60;
    Serial.print("Trenutno min: ");
    Serial.println(now.minute());
    
    Serial.print("Aktiviraj u: ");
    Serial.println(vrijemeAkcije);
  }
  
 if(((now.hour() == onHourAM) && (now.minute() >= onMinuteAM)) || 
    (now.hour() == onHourPM) && (now.minute() >= onMinutePM))     //relax time
    {
     
      fadeLEDon();
      relax=true;
      Serial.print("Relax");
    }
    

 if((now.minute()>30) && (relax==1)){
       fadeLEDoff();
       relax=false;
    }
    
  EthernetClient client = server.available();  // try to get client
  
  if (client) {  // got client?
        boolean currentLineIsBlank = true;
        while (client.connected()) {
            if (client.available()) {   // client data available to read
                char c = client.read(); // read 1 byte (character) from client
                // limit the size of the stored received HTTP request
                // buffer first part of HTTP request in HTTP_req array (string)
                // leave last element in array as 0 to null terminate string (REQ_BUF_SZ - 1)
                if (req_index < (REQ_BUF_SZ - 1)) {
                    HTTP_req[req_index] = c;          // save HTTP request character
                    req_index++;
                }
                // last line of client request is blank and ends with \n
                // respond to client only after last line received
                if (c == '\n' && currentLineIsBlank) {
                    // send a standard http response header
                    client.println("HTTP/1.1 200 OK");
                    // remainder of header follows below, depending on if
                    // web page or XML page is requested
                    // Ajax request - send XML file
                    if (StrContains(HTTP_req, "ajax_inputs")) {
                        // send rest of HTTP header
                        client.println("Content-Type: text/xml");
                        client.println("Connection: keep-alive");
                        client.println();
                        SetLEDs();
                        // send XML file containing input states
                        XML_response(client);
                    }
                    else {  // web page request
                        // send rest of HTTP header
                        client.println("Content-Type: text/html");
                        client.println("Connection: keep-alive");
                        client.println();
                        // send web page
                        webFile = SD.open("index.htm");        // open web page file
                        if (webFile) {
                            while(webFile.available()) {
                                client.write(webFile.read()); // send web page to client
                            }
                            webFile.close();
                        }
                    }
                    // display received HTTP request on serial port
                    Serial.print(HTTP_req);
                    // reset buffer index and all buffer elements to 0
                    req_index = 0;
                    StrClear(HTTP_req, REQ_BUF_SZ);
                    break;
                }
                // every line of text received from the client ends with \r\n
                if (c == '\n') {
                    // last character on line of received text
                    // starting new line with next character read
                    currentLineIsBlank = true;
                } 
                else if (c != '\r') {
                    // a text character was received from client
                    currentLineIsBlank = false;
                }
            } // end if (client.available())
        } // end while (client.connected())
        delay(1);      // give the web browser time to receive the data
        client.stop(); // close the connection
    } // end if (client)
}

// checks if received HTTP request is switching on/off LEDs
// also saves the state of the LEDs
void SetLEDs(void)
{
    // LED 1 (pin 6)
    if (StrContains(HTTP_req, "LED1=1")) {
        LED_state[0] = 1;  // save LED state
        digitalWrite(6, HIGH);
    }
    else if (StrContains(HTTP_req, "LED1=0")) {
        LED_state[0] = 0;  // save LED state
        digitalWrite(6, LOW);
    }
    // LED 2 (pin 7)
    if (StrContains(HTTP_req, "LED2=1")) {
        LED_state[1] = 1;  // save LED state
        digitalWrite(7, HIGH);
    }
    else if (StrContains(HTTP_req, "LED2=0")) {
        LED_state[1] = 0;  // save LED state
        digitalWrite(7, LOW);
    }
    // LED 3 (pin 8)
    if (StrContains(HTTP_req, "LED3=1")) {
        LED_state[2] = 1;  // save LED state
        digitalWrite(8, HIGH);
    }
    else if (StrContains(HTTP_req, "LED3=0")) {
        LED_state[2] = 0;  // save LED state
        digitalWrite(8, LOW);
        motionDetectedString = "Čekam...";
    }
    // LED 4 (pin 9)
    if (StrContains(HTTP_req, "LED4=1")) {
        LED_state[3] = 1;  // save LED state
        digitalWrite(9, HIGH);
    }
    else if (StrContains(HTTP_req, "LED4=0")) {
        LED_state[3] = 0;  // save LED state
        digitalWrite(9, LOW);
    }
}

// send the XML file with analog values, switch status
//  and LED status
void XML_response(EthernetClient cl)
{
  
    cl.print("<?xml version = \"1.0\" ?>");
    cl.print("<inputs>");
    
    cl.print("<vrijeme>");
    
    cl.print(motionDetectedString);
    
    cl.print("</vrijeme>");
    
    cl.print("</inputs>");
}

// sets every element of str to 0 (clears array)
void StrClear(char *str, char length)
{
    for (int i = 0; i < length; i++) {
        str[i] = 0;
    }
}

// searches for the string sfind in the string str
// returns 1 if string found
// returns 0 if string not found
char StrContains(char *str, char *sfind)
{
    char found = 0;
    char index = 0;
    char len;

    len = strlen(str);
    
    if (strlen(sfind) > len) {
        return 0;
    }
    while (index < len) {
        if (str[index] == sfind[found]) {
            found++;
            if (strlen(sfind) == found) {
                return 1;
            }
        }
        else {
            found = 0;
        }
        index++;
    }

    return 0;
}
void fadeLEDon()
{
  for(int i = 0; i <= 255; i++)  //loop to fade LEDs on and off
  {
    analogWrite(ledPin, i);
    delay(10);
  }
}

void fadeLEDoff()
{
  for(int i = 255; i > 0; i--)  //loop to fade LEDs on and off
  {
    analogWrite(ledPin, i);
    delay(10);
  }
  analogWrite(ledPin, 0);
}

void cekajDugo() {
  
}



/*Function to print the current time to serial monitor. Used for debugging*/
void printTime()
{
    now = rtc.now();
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.println(now.second(), DEC);
}

/*function for showing the system is starting by fading and blinking LEDs*/
void initOn()
{
  ledState = fadeLED(HIGH);
  delay(500);
  digitalWrite(ledPin, LOW);
  delay(500);
  digitalWrite(ledPin, HIGH);
  delay(500);
  digitalWrite(ledPin, LOW);
  delay(500);
  digitalWrite(ledPin, HIGH);
  delay(500);
  ledState = fadeLED(LOW);
  delay(500);
  Serial.println("Bed leds initialized");
}

bool fadeLED(bool posInc)
{
  bool ledState;
  int ledVal, incVal;
  if(posInc)
  {
    ledVal = 0; //LEDs starts off and will fade on in for loop
    incVal = 1; //used to increase brightness
    ledState = true;
    Serial.println("Fading on");
  }
  else
  {
    ledVal = 255; //LEDs starts on and will fade off in for loop
    incVal = -1;  //used to decrease brightness
    ledState = false;
    Serial.println("Fading off");
  }
  for(int i = 0; i <= 255; i++)  //loop to fade LEDs on and off
  {
    analogWrite(ledPin, ledVal);
    ledVal += incVal;
    delay(30);
  }
  return ledState;  //returns whether lights are on or off
}
